import Flutter
import UIKit
import Smooch


public class SwiftSmoochPlugin: NSObject, FlutterPlugin {

    var channel : FlutterMethodChannel?

    public static func register(with registrar: FlutterPluginRegistrar) {
        let channel = FlutterMethodChannel(name: "smooch_plugin", binaryMessenger: registrar.messenger())
        let instance = SwiftSmoochPlugin()
        instance.channel = channel
        registrar.addMethodCallDelegate(instance, channel: channel)
    }

    public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        if call.method == "initialize" {
            initialize(call.arguments, result: result)
        } else if call.method == "show" {
            show(call.arguments, result: result)
        } else if call.method == "logout" {
            logout(result: result)
        }
    }

}

// MARK: - FlutterMethodCalls
extension SwiftSmoochPlugin {

    private func initialize(_ arguments: Any?, result: @escaping FlutterResult) {
        if let arguments = arguments as? [String: Any],
            let appId = arguments["appId"] as? String,
            let userId = arguments["userId"] as? String,
            let smoochToken = arguments["smoochToken"] as? String {

            SmoochHandler.shared().app_id = appId
            SmoochHandler.shared().smooch_token = smoochToken

            let settings = SKTSettings(appId: appId)
            settings.enableAppDelegateSwizzling = false
            settings.allowedMenuItems = []
            settings.enableUserNotificationCenterDelegateOverride = false
            settings.conversationStatusBarStyle = UIStatusBarStyle.lightContent;
            Smooch.destroy()

            Smooch.initWith(settings) { error, dictionary in
                if let error = error {
                    result(-1)
                    print(error)
                    return
                }
                let user = "\(userId)"
                Smooch.login(user, jwt: smoochToken, completionHandler: { error, info in
                    SmoochHandler.shared().smoochInited = true
                    SmoochHandler.shared().conversation = Smooch.conversation()
                })
            }
        }
    }

    private func show(_ arguments: Any?, result: @escaping FlutterResult) {
        if let arguments = arguments as? [String: Any],
            let properties = arguments["properties"] as? [String: Any] {
            SmoochHandler.shared().properties = properties
        }
        SmoochHandler.shared().show(channel) { (res) in
            result(res)
        }
    }

    private func logout(result: @escaping FlutterResult) {
        Smooch.logout(completionHandler: nil)
    }

}
