import 'dart:async';

import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:smooch_plugin/smooch_status.dart';

class SmoochPlugin {
  static const MethodChannel _channel = const MethodChannel('smooch_plugin');

  static Future<SmoochStatus> init(
      {@required String appId, @required String userId, @required String smoochToken, onUpdateUnreadCount, onHandleMessageAction, onShouldSendStart, onLogout}) async {
    _initializeChannel(onUpdateUnreadCount: onUpdateUnreadCount, onHandleMessageAction: onHandleMessageAction, onShouldSendStart: onShouldSendStart, onLogout: onLogout);

    final int result = await _channel.invokeMethod('init', <String, dynamic>{
      'appId': appId,
      'userId': userId,
      'smoochToken': smoochToken,
    });

    if (result == -1) {
      return SmoochStatus.not_initialized;
    } else if (result == 0) {
      return SmoochStatus.successful;
    }

    return null;
  }

  static Future<SmoochStatus> show(String lang, [Map<String, dynamic> userProperties]) async {
    final int result = await _channel.invokeMethod('show', <String, dynamic>{
      'lang': lang,
      'properties': userProperties
    });
    if (result == -1) {
      return SmoochStatus.not_initialized;
    } else if (result == 0) {
      return SmoochStatus.successful;
    }

    return null;
  }

  static Future<bool> logout() async {
    await _channel.invokeMethod("logout");

    return true;
  }

  static void _initializeChannel({onUpdateUnreadCount, onHandleMessageAction, onShouldSendStart, onLogout}) {
    _channel.setMethodCallHandler((MethodCall call) async {
      String method = call.method;
      dynamic arguments = call.arguments;

      if (method == "logoutComplete") {
        if (onLogout != null) {
          onLogout();
        }
      } else if (method == "updateChatUnreadCount") {
        if (onUpdateUnreadCount != null) {
          onUpdateUnreadCount(arguments);
        }
      } else if (method == "handleMessageAction") {
        if (onHandleMessageAction != null) {
          onHandleMessageAction(arguments);
        }
      } else if (method == "shouldSendStart") {
        if (onShouldSendStart != null) {
          onShouldSendStart();
        }
      }
    });
  }
}
